/*
   File: affix_rules.h
   Checks the affix rules

   Copyright (C) 2008 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: affix_rules.h,v 1.13 2010/06/12 12:54:23 marcs Exp $"
*/
#ifndef IncAffixRules
#define IncAffixRules

/* standard includes */
#include <stdio.h>

/* libdcg includes */
#include <dcg.h>
#include <dcg_alloc.h>
#include <dcg_string.h>

/* local includes */
#include "cdl_ds.h"

/* Exported routines to create anonymous affix rule names */
string new_anonymous_affix_name ();
void init_affix_rules ();

/* One exported constant */
#define AFFIX_ERROR ((affix_rule) (1L))

typedef enum prim_types 
{ prim_none, prim_int, prim_real, prim_text } prim_type;

/* Exported functions */
void check_affix_rules ();
affix_rule get_prime_affix_rule (affix_rule arule);
affix_rule get_repr_affix_rule (affix_rule arule);
string get_affix_rule_name (affix_rule arule);
affix_alternative_list pick_rhs_affix_alts (affix_rule arule);

int is_abstract_synonym (affix_rule arule, affix_rule *orig_arule, affix_rule *rhs);
int equivalent_affix_rule (affix_rule arule1, affix_rule arule2);
int equivalent_signatures (affix_rule_list arules1, affix_rule_list arules2);
affix_rule identify_affix_instance (string vname, module mod);

int is_integer_type (affix_rule arule);
int is_real_type (affix_rule arule);
int is_text_type (affix_rule arule);
int is_a_primitive_type (affix_rule arule);
affix_rule get_primitive_type (prim_type pt);

#endif /* IncAffixRules */
