/*
   File: backend.c
   Finishes the compilation by assembling and linking the code

   Copyright (C) 2008-2010 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: backend.c,v 1.8 2012/01/20 09:41:48 marcs Exp $"
*/

/* include <config.h> if autoconfigured */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

/* global includes */
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include <sys/types.h>

/* libdcg includes */
#include <dcg.h>
#include <dcg_error.h>
#include <dcg_string.h>
#include <dcg_procutil.h>

/* local includes */
#include "options.h"

/* TMPDIR is the usual environment variable to consult */ 
#ifndef TMPDIR
#define TMPDIR "TMPDIR"
#endif

/* Define the usual default temporary directory */
#ifndef DEFAULT_TMP_DIR
#define DEFAULT_TMP_DIR "/tmp"
#endif

/* Define the default include directory */
#ifndef INCDIR
#define INCDIR "/usr/local/share/mcdl/include"
#endif

#ifndef WIN32
static void do_compile ()
{ char *cc_argv[12];
  int status;
  int ix = 0;

  /* Define the name of the object file */
  if (!generate_object)
    { /* We do not want to leave the object code in place */
      char *temp_dir = getenv (TMPDIR);
      if (temp_dir == NULL) temp_dir = DEFAULT_TMP_DIR;
      object_fname = dcg_new_fmtd_string ("%s%cmcdl_%d.o", temp_dir, DIR_SEP, (int) getpid ());
    }
  else if (output_fname != NULL)        /* -o name given */
    object_fname = output_fname;
  else object_fname = dcg_new_fmtd_string ("%s.o", base_mname);

  /* Fill in the argument vector to assemble */
  dcg_warning (0, "   compiling generated C code...");
  cc_argv[ix++] = "gcc";
  cc_argv[ix++] = "-I";
  cc_argv[ix++] = INCDIR;
  cc_argv[ix++] = "-c";
  cc_argv[ix++] = c_code_fname;
  cc_argv[ix++] = "-O";
  cc_argv[ix++] = "-o";
  cc_argv[ix++] = object_fname;
  cc_argv[ix] = NULL;
  status = dcg_spawn_and_wait (cc_argv);

  /* Delete the generated c_code file */
  if (unlink (c_code_fname))
    dcg_panic ("could not delete '%s'", c_code_fname);
  if (status) dcg_exit (4);
}

static void do_link ()
{ char **ld_argv;
  char *exe_name;
  int nr_args = 3 * library_search_path -> size + link_library_names -> size + 12;
  int status;
  int ix = 0;
  int iy;
  ld_argv = (char **) dcg_calloc (nr_args, sizeof (char *));

  /* Define the name of the executable */
  if (output_fname != NULL)             /* -o name given */
    exe_name = output_fname;
  else exe_name = base_mname;

  /* Fill in the argument vector to link */
  dcg_warning (0, "   linking object code...");
  ld_argv[ix++] = "gcc";
  ld_argv[ix++] = "-o";
  ld_argv[ix++] = exe_name;
  ld_argv[ix++] = object_fname;
  for (iy = 0; iy < library_search_path -> size; iy++)
    { ld_argv[ix++] = dcg_new_fmtd_string ("-L%s", library_search_path -> array[iy]);
      ld_argv[ix++] = "-Wl,-rpath";
      ld_argv[ix++] = dcg_new_fmtd_string ("-Wl,%s", library_search_path -> array[iy]);
    };
  for (iy = 0; iy < link_library_names -> size; iy++)
    ld_argv[ix++] = dcg_new_fmtd_string ("-l%s", link_library_names -> array[iy]);
  ld_argv[ix] = NULL;
  status = dcg_spawn_and_wait (ld_argv);

  /* Delete the generated object file */
  if (unlink (object_fname))
    dcg_panic ("could not delete '%s'", object_fname);
  if (status) dcg_exit (4);
}
#endif

void compile_c_and_link ()
{ /* If we only have to generate assembler, we are done */
  if (partial_compilation || generate_c_code) return;
#ifndef WIN32
  do_compile ();
  if (generate_object) return;
  do_link ();
#endif
}

