/*
   File: contsens.c
   Does context dependent analysis

   Copyright (C) 2008 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: contsens.c,v 1.20 2012/04/21 14:00:00 marcs Exp $"
*/

/* global includes */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

/* libdcg includes */
#include <dcg.h>
#include <dcg_error.h>
#include <dcg_string.h>

/* local includes */
#include "cdl_ds.h"
#include "options.h"
#include "globals.h"
#include "modules.h"
#include "incr_error.h"
#include "affix_rules.h"
#include "procedures.h"
#include "variables.h"
#include "contsens.h"

/* error administration */
void contsens_error (module mod, int line, int col, char *format, ...)
{ char buf[MAXSTRLEN];
  va_list arg_ptr;
  va_start (arg_ptr, format);
  vsprintf (buf, format, arg_ptr);
  va_end (arg_ptr);
  if (!line && !col) dcg_error (0, "File %s: %s", mod -> fname, buf);
  else dcg_error (0, "File %s, line %d, col %d: %s", mod -> fname, line, col, buf);
  maybe_enter_editor (mod -> fname, line, col);
}

void contsens_error_by_mnr (int mnr, int line, int col, char *format, ...)
{ char buf[MAXSTRLEN];
  va_list arg_ptr;
  va_start (arg_ptr, format);
  vsprintf (buf, format, arg_ptr);
  va_end (arg_ptr);
  if (!line && !col) dcg_error (0, "File %s: %s", all_modules -> array[mnr] -> fname, buf);
  else dcg_error (0, "File %s, line %d, col %d: %s",
		  all_modules -> array[mnr] -> fname, line, col, buf);
  maybe_enter_editor (all_modules -> array[mnr] -> fname, line, col);
}

void contsens_warning (module mod, int line, int col, char *format, ...)
{ char buf[MAXSTRLEN];
  va_list arg_ptr;
  va_start (arg_ptr, format);
  vsprintf (buf, format, arg_ptr);
  va_end (arg_ptr);
  if (!line && !col) dcg_warning (0, "File %s: %s", mod -> fname, buf);
  else dcg_warning (0, "File %s, line %d, col %d: %s", mod -> fname, line, col, buf);
}

void contsens_warning_by_mnr (int mnr, int line, int col, char *format, ...)
{ char buf[MAXSTRLEN];
  va_list arg_ptr;
  va_start (arg_ptr, format);
  vsprintf (buf, format, arg_ptr);
  va_end (arg_ptr);
  if (!line && !col) dcg_warning (0, "File %s: %s", all_modules -> array[mnr] -> fname, buf);
  else dcg_warning (0, "File %s, line %d, col %d: %s",
		    all_modules -> array[mnr] -> fname, line, col, buf);
}

void contsens_hint (module mod, int line, int col, char *format, ...)
{ char buf[MAXSTRLEN];
  va_list arg_ptr;
  va_start (arg_ptr, format);
  vsprintf (buf, format, arg_ptr);
  va_end (arg_ptr);
  if (!line && !col) dcg_hint ("File %s: %s", mod -> fname, buf);
  else dcg_hint ("File %s, line %d, col %d: %s", mod -> fname, line, col, buf);
}

void contsens_hint_by_mnr (int mnr, int line, int col, char *format, ...)
{ char buf[MAXSTRLEN];
  va_list arg_ptr;
  va_start (arg_ptr, format);
  vsprintf (buf, format, arg_ptr);
  va_end (arg_ptr);
  if (!line && !col) dcg_hint ("File %s: %s", all_modules -> array[mnr] -> fname, buf);
  else dcg_hint ("File %s, line %d, col %d: %s",
		 all_modules -> array[mnr] -> fname, line, col, buf);
}

void pass2_error (module mod, int line, int column)
{ if (!line && !column)
    dcg_error (0, "File %s: second pass constructs are not implemented", mod -> fname);
  else
    dcg_error (0, "File %s, line %d, col %d: second pass constructs are not implemented",
	       mod -> fname, line, column);
}

/*
   Context sensitive analysis
*/
void context_sensitive_analysis ()
{ check_modules ();
  check_affix_rules ();
  check_global_variables ();
  check_procedures ();
  if (dump_checker)
    { dump_modules ();
      dcg_wlog ("PRELUDE: ");
      pp_proc (dcg_error_file (), root_module -> Root_module.prelude);
      dcg_wlog ("");
      dcg_wlog ("ROOT: ");
      pp_proc (dcg_error_file (), root_module -> Root_module.root);
      dcg_wlog ("");
      dcg_wlog ("POSTLUDE: ");
      pp_proc (dcg_error_file (), root_module -> Root_module.postlude);
      dcg_wlog ("");
    };
};
