/*
   File: main.c
   Driver for mcdl compiler

   Copyright (C) 2008-2010 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: main.c,v 1.14 2011/09/02 10:11:23 marcs Exp $"
*/

/* global includes */
#include <stdio.h>

/* libdcg includes */
#include <dcg.h>
#include <dcg_error.h>
#include <dcg_alloc.h>

/* local includes */
#include "cdl_ds.h"
#include "options.h"
#include "globals.h"
#include "backend.h"
#include "affix_rules.h"
#include "contfree.h"
#include "contsens.h"
#include "intermed.h"
#include "optimize.h"
#include "coder.h"
#include "flow.h"

static void init_mcdl_compiler ()
{ dcg_init_error (stderr, "mcdl", NULL);
  dcg_init_alloc ("mcdl", 0);
  init_options ();
  init_globals ();
  init_affix_rules ();
}

static void compile_source ()
{ context_free_analysis ();
  context_sensitive_analysis ();
  flow_analysis ();
  intermediate_code_generation ();
  intermediate_code_optimization ();
  target_code_generation ();
  compile_c_and_link ();
}

int main (int argc, char **argv)
{ init_mcdl_compiler ();
  parse_command_line (argc, argv);
  compile_source ();
  return (0);
}
