/*
   File: modules.c
   Maintains and checks the list of all modules.

   Copyright (C) 2008 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: modules.c,v 1.9 2012/04/21 14:00:00 marcs Exp $"
*/

/* standard includes */
#include <stdio.h>
#include <string.h>

/* libdcg includes */
#include <dcg.h>
#include <dcg_alloc.h>
#include <dcg_error.h>
#include <dcg_string.h>

/* local includes */
#include "cdl_ds.h"
#include "globals.h"
#include "modules.h"

/* Module addition and lookup */
void register_module (module mod)
{ mod -> mnr = all_modules -> size;
  app_module_list (all_modules, mod);
}

module identify_module (string mname)
{ int ix;
  for (ix = 0; ix < all_modules -> size; ix++)
    if (streq (mname, all_modules -> array[ix] -> mname))
      return (all_modules -> array[ix]);
  return (module_nil);
}

/* Checks on modules */
void check_modules ()
{ int ix;
  dcg_warning (0, "   checking modules...");

  /* Check that we have no multiple root modules */
  for (ix = 0; ix < all_modules -> size; ix++)
    { module mod = all_modules -> array[ix];
      if (mod -> tag == TAGRoot_module)
	{ if (root_module == module_nil) root_module = mod;
	  else dcg_error (1, "Multiple root module %s", mod -> mname);
	};
    };

  /* Check that we have a root module */
  if (root_module == module_nil)
    dcg_error (1, "No root module found");

  /* Finish up */
  dcg_panic_if_errors ();
  dcg_hint ("      collected %d modules to compile", all_modules -> size);
}

void init_modules ()
{ root_module = module_nil;
  all_modules = new_module_list ();
}

void dump_modules ()
{ dcg_wlog ("List of all modules:");
  pp_module_list (dcg_error_file (), all_modules);
  dcg_wlog ("");
};
