/*
   File: proc_tree.h
   Handles trees of declarations

   Copyright (C) 2008 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: proc_tree.h,v 1.5 2011/09/05 19:56:22 marcs Exp $"
*/
#ifndef IncProcTree
#define IncProcTree

/* standard includes */
#include <stdio.h>

/* libdcg includes */
#include <dcg.h>
#include <dcg_alloc.h>
#include <dcg_string.h>
#include <dcg_dump.h>

/* local includes */
#include "cdl_ds.h"

/* introduce type and nil definitions */
typedef struct proc_tree_rec *proc_tree;
struct proc_tree_rec
{ string_list tag;
  pdecl_list pdefs;
  proc_tree left;
  proc_tree right;
};
#define proc_tree_nil ((proc_tree) NULL)

/* introduce necessary routines and shorthands */
#define new_proc_tree() proc_tree_nil
#define attach_proc_tree(orig) (proc_tree) dcg_attach ((char *) orig)
#define att_proc_tree(orig) (void) dcg_attach ((char *) orig)
void detach_proc_tree (proc_tree *optr);
#define det_proc_tree(optr) detach_proc_tree(optr)
#define equal_proc_tree(a,b) ((a) == (b))

/* supplementary routines and shorthands */
#define est_proc_tree(x) MAXWIDTH
void ppp_proc_tree (FILE *f, int horiz, int ind, proc_tree old);
#define pp_proc_tree(f,x) ppp_proc_tree (f,0,0,x)
void save_proc_tree (FILE *f, proc_tree old);
int load_proc_tree (FILE *f, proc_tree *x);

/* specific routines and shorthands */
pdecl_list enter_proc_tree (proc_tree *root, string_list pname_parts);
pdecl_list lookup_proc_tree (proc_tree root, string_list pname_parts);

#endif /* IncProcTree */
