/*
   File: mrts_mcdl.h
   Defines the basic MCDL runtime system

   Copyright (C) 2008 Marc Seutter

   This library is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published   
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
   along with this library.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: mrts_mcdl.h,v 1.14 2009/02/18 14:53:44 marcs Exp $"
*/
#ifndef IncMrtsMcdl
#define IncMrtsMcdl

/* System includes */
#include <stdio.h>

/* For compiling with C++ */
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
   Our basic value is a monster with many faces:

   INT: value itself (long)
   REAL:  "      "   (double)
   else: pointer to the heap (NULL representing the undefined value),
	 pointing to a combination of a refcount, an alternative number
	 and a TEXT/FILE/other primitive singleton (add offset 1 * size)
	 or a row of values
*/
typedef union v_uni *vptr;
typedef union v_heap_uni *heap_vptr;
typedef union v_uni
{ long		lv;	/* INT */
  double	re;	/* REAL */
  heap_vptr	vp;	/* Access on members */
} value;

typedef struct s_rts_adm
{ int ref_count;
  int tag;
} rts_adm;

union v_heap_uni
{ value	vpart;		/* a part of a composite type */
  char *vtext;		/* a pointer to a TEXT */
  FILE *vfile;		/* a pointer to a file */
  rts_adm adm;		/* Runtime administration */
  heap_vptr next;	/* Free list link */
};

/*
   Access macros
*/
#define Admin(r,t)	 { (r), (t) }
#define Refcount(v)	 ((v).vp -> adm.ref_count)
#define AlternativeNr(v) ((v).vp -> adm.tag)
#define Vptr(v)		 ((v).vp)
#define Part(v,i)	 ((v).vp[i])
#define PartValue(v,i)	 ((v).vp[i].vpart)
#define InitValue	 { 0L }

#ifndef WIN32
#define ValueCast(vp)	 ((value)(vp))
#else
__inline value ValueCast (heap_vptr vp) { value v; v.vp = vp; return (v); }
#endif

/*
   Attaching and Detaching
*/
#ifdef MEMCHECK
#define TryDecrRefcount(v)	mrts_try_decr_refcount ((v))
#define Attach(v)		mrts_attach_value ((v))
#else
#define TryDecrRefcount(v)	((((v).vp) != NULL) && (Refcount(v) > 0) && ((--Refcount(v)) == 0))
#define Attach(v)		if ((((v).vp) != NULL) && (Refcount(v) > 0)) (Refcount(v)++)
#endif
#define Free(v,s)		mrts_free_value ((v), (s))

/*
   Text support
*/
#define SizeFromLength(s)	(((s) + sizeof(value) - 1) / sizeof(value))

/*
   Array definition
*/
typedef struct array_rec
{ void *bptr;
  int depth;
} Array;
#define InitArray	{ NULL, 0 }

/*
   Constant initializer definition
*/
typedef struct const_init_rec
{ int const_nr;
  int tag;
  char *ctext;
} const_init;

/*
   Exported runtime routines
*/
void mrts_panic (char *format, ...);
void mrts_error (char *format, ...);
void mrts_abort (char *module_name, int linenr);
void mrts_exit (void);
heap_vptr ck_walloc (int nr_words);
heap_vptr mrts_alloc_value (int nr_words);
value mrts_create_value (int tag, int nr_words);
int mrts_try_decr_refcount (value v);
void mrts_attach_value (value v);
void mrts_free_value (value old, int nr_words);
void mrts_ucfree_value (value old, int nr_words);
value *mrts_index_array (Array *g_array, long idx);
void mrts_init_constants (int nr, const_init *table, value *consts);
void mrts_init_basic ();

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* IncMrtsMcdl */
