/*
   File: mrts_stdtypes.c
   Defines basic operations for standard types

   Copyright (C) 2008 Marc Seutter

   This library is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published   
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
   along with this library.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: mrts_stdtypes.c,v 1.11 2010/03/19 09:18:20 marcs Exp $"
*/

/* Standard includes */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

/* Local includes */
#include "mrts_mcdl.h"
#include "mrts_stdtypes.h"

/*
   TEXT operations
*/
value mrts_alloc_text (char *vtext)
{ value v = mrts_create_value (Tag_TEXT, 2);
  Part(v,1).vtext = vtext;
  return (v);
}

value mrts_create_text (char *tv)
{ int len = (int) strlen (tv);
  char *vtext = (char *) mrts_alloc_value (SizeFromLength (len + 1));
  strcpy (vtext, tv);
  return (mrts_alloc_text (vtext));
}

void mrts_free_text (value v)
{ int len = (int) strlen (Part(v,1).vtext);
  mrts_ucfree_value (Part(v,1).vpart, SizeFromLength (len + 1));
  mrts_free_value (v, 2);
}

value mrts_concat_text (value v1, value v2)
{ int nlen = (int) (strlen (Part(v1,1).vtext) + strlen (Part(v2,1).vtext));
  char *vtext = (char *) mrts_alloc_value (SizeFromLength (nlen + 1));
  value v = mrts_create_value (Tag_TEXT, 2);
  char *sptr, *dptr;

  Part(v,1).vtext = vtext;
  for (sptr = Part(v1,1).vtext, dptr = vtext; *sptr; sptr++, dptr++) *dptr = *sptr;
  for (sptr = Part(v2,1).vtext; *sptr; sptr++, dptr++) *dptr = *sptr;
  *dptr = '\0';
  return (v);
}

value mrts_concat_texts (value *vl, int nr)
{ value v = mrts_create_value (Tag_TEXT, 2);
  char *vtext, *sptr, *dptr;
  int nlen = 0;
  int ix;
  for (ix = 0; ix < nr; ix++) nlen += (int) strlen (Part(vl[ix],1).vtext);
  vtext = (char *) mrts_alloc_value (SizeFromLength (nlen + 1));
  Part(v,1).vtext = vtext;
  dptr = vtext;
  for (ix = 0; ix < nr; ix++)
    for (sptr = Part(vl[ix],1).vtext; *sptr; sptr++, dptr++) *dptr = *sptr;
  *dptr = '\0';
  return (v);
}

value mrts_repl_text (long lv, value v2)
{ value v = mrts_create_value (Tag_TEXT, 2);
  char *vtext, *sptr, *dptr;
  int srclen = (int) strlen (Part(v2,1).vtext);
  int ix;

  if (lv < 0L)
    mrts_error ("Cannot multiply %ld with TEXTs", lv);
  vtext = (char *) mrts_alloc_value (SizeFromLength (lv * srclen + 1));
  Part(v,1).vtext = vtext;
  dptr = vtext;
  for (ix = 0; ix < lv; ix++)
    for (sptr = Part(v2,1).vtext; *sptr; sptr++, dptr++) *dptr = *sptr;
  *dptr = '\0';
  return (v);
}

/*
   FILE operations
*/
value mrts_create_file (FILE *fv)
{ value v = mrts_create_value (Tag_FILE, 2);
  Part(v,1).vfile = fv;
  return (v);
}

void mrts_free_file (value v)
{ /* fclose (v)? */
  mrts_free_value (v, 2);
}
